/*
	     .o        88   oooooooo      88  .ooooo.    ooooooooo
	   .d88       .8'  dP"""""""     .8' 888' `Y88. d"""""""8'
	 .d'888      .8'  d88888b.      .8'  888    888       .8'
       .d'  888     .8'       `Y88b    .8'    `Vbood888      .8'
       88ooo888oo  .8'          ]88   .8'          888'     .8'
	    888   .8'     o.   .88P  .8'         .88P'     .8'
	   o888o  88      `8bd88P'   88        .oP'       .8'

     []-------------------------------------------------------------[]
     |		                CIRC3D.C                             |
     |						 	             |
     |         Teste de caminhada sobre uma curva no espaco          |
     |						                     |
     |                por Lenimar N. Andrade, maio/97                |
     []-------------------------------------------------------------[]

Para compilar:
   tcc -2 -DHELICE -e3d1.exe circ3d.c graphics.lib
   tcc -2 -DCURVA3D1 -e3d2.exe circ3d.c graphics.lib
   tcc -2 -DCURVA3D2 -e3d3.exe circ3d.c graphics.lib
   tcc -2 -DVALORES -DPAUSA -DHELICE -e_3d1.exe circ3d.c graphics.lib
   tcc -2 -DVALORES -DPAUSA -DCURVA3D1 -e_3d2.exe circ3d.c graphics.lib
   tcc -2 -DVALORES -DPAUSA -DCURVA3D2 -e_3d3.exe circ3d.c graphics.lib

*/

#define GRAF3D
#include "graf.c"

#define COR_CURVA             DARKGRAY     
#define COR_CIRC              CYAN         
#define COR_PONTOS_NA_CURVA   RED           
#define COR_PASSO_TANGENCIAL  LIGHTMAGENTA  
#define COR_PASSO_CIRCULAR    WHITE         

/* #define HELICE */
/* #define CURVA3D1 */
/* #define CURVA3D2 */
/* #define CURVA3D3 */
/* #define CURVA3D4 */

/* #define PAUSA */
/* #define VALORES */
#define DEMORA

#ifdef PAUSA
#undef DEMORA
#endif

#ifdef HELICE
double f1(double t) { return 1.5*cos(t); }
double f2(double t) { return 1.5*sin(t); }
double f3(double t) { return t/4; }
#endif

#ifdef CURVA3D1
double f1(double t) { return 2*sin(2*t); }
double f2(double t) { return 2*sin(t)*cos(2*t); }
double f3(double t) { return 2*cos(t)*cos(2*t); }
#endif

#ifdef CURVA3D2
double f1(double t) { return (5 + 2*sin(20*t))*cos(t); }
double f2(double t) { return (5 + 2*sin(20*t))*sin(t); }
double f3(double t) { return 2*cos(20*t); }
#endif

#ifdef CURVA3D3
double f1(double t) { return (5 + 2*sin(5*t))*cos(2*t); }
double f2(double t) { return (5 + 2*sin(5*t))*sin(2*t); }
double f3(double t) { return 2*cos(5*t); }
#endif

#ifdef CURVA3D4
double f1(double t) { return 1 + cos(t); }
double f2(double t) { return sin(t); }
double f3(double t) { return 2*sin(t/2); }
#endif

/* ----------------------------------------------------------------------- */

void DesenhaCirc3D(ponto P, vetor v1, ponto Q, vetor v2, ponto C,
						      double incr, int cor) {

/* Desenha uma circunferencia de centro C e que passa por Q no plano
   que contem os vetores v1 e v2 */

  double teta;
  double proj_X, proj_Y;
  ponto R;

  for (teta = 0; teta <= DOIS_PI; teta += incr)
    if (PontoNaCircunferencia(&R, P, v1, Q, v2, C, teta)) {
      ProjetaPonto(R.x, R.y, R.z, &proj_X, &proj_Y);
      putpixel(A + proj_X*TAM, B - proj_Y*TAM, cor);
    }
}

/* ----------------------------------------------------------------------- */

void main(int argc, char *argv[]) {

  ponto C, R, T, P, Q;
  vetor v1, v2, v;
  char ch, equacoes[100], mens[100], nome_arq[30] = "DADOS.TXT",
       SALVAR_ARQ_TEXTO = 1;
  double torcao, k, norma_v, n, incr, t1, t2, tmin, tmax, r, distR,
	 distT, tT, tR,
	 incrC,           /* Incr. no parametro da circunferencia   */
	 incr3D,          /* Incremento no parametro da curva       */
	 t_incr = 0.1,    /* Intervalo entre t1 e t2 (na curva)     */
	 d_incr = 0.01,   /* Usado para calcular distancia `a curva */
	 L = 0.3;         /* Tamanho do passo da caminhada          */
  int eixos = 1;          /* Define se desenha os eixos x, y, z     */
  FILE *arq;
  #ifdef DEMORA
  int demora = 200;       /* Milisegundos entre duas amostragens    */
  if (argc > 1) demora = atoi(argv[1]);
  #endif
  
  if ((arq = fopen(nome_arq, "wt")) == NULL) {
    printf("ERRO: Arquivo %s nao pode ser aberto", nome_arq);
    return;
  }

  #ifdef HELICE
  strcpy(equacoes, "f(t) = (3/2 cos t, 3/2 sen t, t/4)");
  TAM = 60;
  tmin = -12, tmax = 12;
  n = 30;
  incr3D = 0.01;
  incrC =  0.02;
  #endif

  #ifdef CURVA3D1
  strcpy(equacoes, "f(t) = (2 sen(2t), 2sen(t)cos(2t), 2cos(t)cos(2t))");
  TAM = 90;
  tmin = 0, tmax = 2*PI;
  n = 100;
  eixos = 0;
  fi = -PI/13, teta = -PI/5;
  incr3D = 0.01;
  incrC =  0.01;
  #endif

  #ifdef CURVA3D2
  strcpy(equacoes, "f(t) = ((5+2*sin(20*t))*cos(t), (5+2*sin(20*t))*sin(t),"
		   " 2*cos(20*t))");
  TAM = 40;
  tmin = 0, tmax = 2*PI;
  n = 70;
  eixos = 0;
  fi = -PI/13, teta = -PI/5;
  incr3D = 0.002;
  incrC =  0.01;
  #endif

  #ifdef CURVA3D3
  strcpy(equacoes, "f(t) = ((5+2*sin(5*t))*cos(2*t), "
			     "(5+2*sin(5*t))*sin(2*t), 2*cos(5*t))");
  TAM = 40;
  tmin = 0, tmax = 2*PI;
  n = 100;
  eixos = 0;
  fi = -PI/13, teta = -PI/5;
  incr3D = 0.005;
  incrC =  0.01;
  #endif

  #ifdef CURVA3D4
  strcpy(equacoes, "f(t) = (1 + cos t, sen t, 2 sen t/2)");
  TAM = 80;
  tmin = -2*PI, tmax = 2*PI;
  n = 100;
  eixos = 0;
  fi = -PI/13, teta = -PI/5;
  incr3D = 0.02;
  incrC =  0.02;
  #endif

  incr = (tmax - tmin)/n;
  t1 = tmin;
  t2 = t1 + t_incr;
  fprintf(arq, "%s\n", equacoes);
  fprintf(arq, "L = %5.5f\n", L);

  if (!IniciaModoGrafico()) {
    printf("\nErro na mudanca para modo grafico\n");
    return;
  }

  do {

    P.x = f1(t1), P.y = f2(t1), P.z = f3(t1);
    v1.x = DER(f1,t1,h_erro), v1.y = DER(f2,t1,h_erro),
    v1.z = DER(f3,t1,h_erro);
    Q.x = f1(t2), Q.y = f2(t2), Q.z = f3(t2);
    v2.x = DER(f1,t2,h_erro), v2.y = DER(f2,t2,h_erro),
    v2.z = DER(f3,t2,h_erro);

    k = 1/Curvatura(f1, f2, f3, t2);
    torcao = Torcao(f1, f2, f3, t2);

    if (!CentroDaCircunferencia(&C, &r, P, v1, Q, v2)) {
      printf("\nErro no calculo do centro da circunferencia\n");
      return;
    }

    if (r < 1) {
      if (!PontoNaCircunferencia(&R, P, v1, Q, v2, C, L)) {
	printf("\nErro no calculo do ponto sobre a circunferencia\n");
	return;
      }
    } else {
      if (!PontoNaCircunferencia(&R, P, v1, Q, v2, C, L/r)) {
	printf("\nErro no calculo do ponto sobre a circunferencia\n");
	return;
      }
    }

    VetorTangenteACurva(&v, f1, f2, f3, t2);
    norma_v = NORMA(v.x, v.y, v.z);
    v.x /= norma_v, v.y /= norma_v, v.z /= norma_v;
    T.x = Q.x + L*v.x, T.y = Q.y + L*v.y, T.z = Q.z + L*v.z;

    distR = Dist(R, f1, f2, f3, tmin, tmax, d_incr, &tR);
    distT = Dist(T, f1, f2, f3, tmin, tmax, d_incr, &tT);

    DesenhaCurva3D(f1, f2, f3, tmin, tmax, incr3D, COR_CIRC);
    DesenhaCirc3D(P, v1, Q, v2, C, incrC, COR_CURVA);
    if (eixos) DesenhaEixos(2, BLUE, BLUE);

    MarcaPonto3D(C.x, C.y, C.z, COR_CURVA);
    MarcaPonto3D(P.x, P.y, P.z, COR_PONTOS_NA_CURVA);
    MarcaPonto3D(Q.x, Q.y, Q.z, COR_PONTOS_NA_CURVA);
    MarcaPonto3D(R.x, R.y, R.z, COR_PASSO_CIRCULAR);
    MarcaPonto3D(T.x, T.y, T.z, COR_PASSO_TANGENCIAL);

    #ifdef VALORES
    setcolor(MAGENTA);
    sprintf(mens, "dist(Tangencial) = %5.5f", distT);
    outtextxy(0, 0, mens);
    setcolor(COR_PASSO_CIRCULAR);
    sprintf(mens, "dist(Circular) = %5.5f", distR);
    outtextxy(0, 10, mens);
    setcolor(COR_CURVA);
    sprintf(mens, "R = %5.5f  1/k = %5.5f", r, k);
    outtextxy(0, 20, mens);
    sprintf(mens, "t = %5.5f  Torcao = %5.5f", t2, torcao);
    setcolor(COR_PONTOS_NA_CURVA);
    outtextxy(0, 30, mens);
    #endif

    if (SALVAR_ARQ_TEXTO) {
      fprintf(arq, "\ndist(Tangencial) = %5.5f", distT);
      fprintf(arq, "\ndist(Circular) = %5.5f", distR);
      fprintf(arq, "\nR = %5.5f  1/k = %5.5f", r, k);
      fprintf(arq, "\nt = %5.5f  Torcao = %5.5f\n", t2, torcao);
    }

    #ifdef DEMORA
    if (kbhit()) {
      closegraph();
      fclose(arq);
      return;
    }
    delay(demora);
    #endif

    #ifdef PAUSA
    ch = getch();
    if (ch == 0) ch = getch();
    else if (ch == 27) {
      closegraph();
      fclose(arq);
      return;
    }
    #endif

    /* Apaga o grafico desenhado anteriormente */
    setcolor(0);
    DesenhaCirc3D(P, v1, Q, v2, C, incrC, 0);
    MarcaPonto3D(C.x, C.y, C.z, 0);
    MarcaPonto3D(P.x, P.y, P.z, 0);
    MarcaPonto3D(Q.x, Q.y, Q.z, 0);
    MarcaPonto3D(R.x, R.y, R.z, 0);
    MarcaPonto3D(T.x, T.y, T.z, 0);

    #ifdef VALORES
    sprintf(mens, "dist(Tangencial) = %5.5f", distT);
    outtextxy(0, 0, mens);
    sprintf(mens, "dist(Circular) = %5.5f", distR);
    outtextxy(0, 10, mens);
    sprintf(mens, "R = %5.5f  1/k = %5.5f", r, k);
    outtextxy(0, 20, mens);
    sprintf(mens, "t = %5.5f  Torcao = %5.5f", t2, torcao);
    outtextxy(0, 30, mens);
    #endif

    t1 += incr;
    t2 += incr;
    if (t2 > tmax) SALVAR_ARQ_TEXTO = 0;

  } while (!kbhit());

  closegraph();
  fclose(arq);
}

/*** FIM DE CIRC3D.C ***/
